#version 330

layout(location = 0) in vec3 position;
layout(location = 1) in vec3 normal;
layout(location = 2) in vec2 uv; 

out vec3 worldNormal;
out vec2 tex_uv;
out vec3 cameraNormal; 
out vec3 cameraLightDirection; 
out vec4 shadowCoord;
out vec4 projTexCoord;
out vec4 clipSpacePos;
out vec4 prevClipSpacePos;
out vec3 cameraLightDirectionSpot; 
out vec3 spotDir;

uniform mat4 projectionMatrix;
uniform mat4 modelMatrix;
uniform mat4 viewMatrix;
uniform mat4 prevModelMatrix;
uniform mat4 prevViewMatrix;

uniform vec3 worldLightPosition;

uniform mat4 depthBiasVP;
uniform mat4 projTexView;
uniform mat4 projTexProjection;

void main()
{
	tex_uv = uv;
	worldNormal = (modelMatrix * vec4(normal,0)).xyz;
	mat4 MVP = projectionMatrix * viewMatrix * modelMatrix;
	
	clipSpacePos = MVP * vec4(position, 1);	
	prevClipSpacePos = projectionMatrix * prevViewMatrix * prevModelMatrix * vec4(position, 1.0);	

	shadowCoord = depthBiasVP * modelMatrix * vec4(position, 1);

	projTexCoord = projTexProjection * projectionMatrix * projTexView * modelMatrix * vec4(position, 1);

	vec3 cameraVertexPosition = ( viewMatrix * modelMatrix * vec4(position,1)).xyz;
	vec3 cameraEyeDirection = vec3(0,0,0) - cameraVertexPosition;
 
	vec3 cameraLightPosition = ( viewMatrix * vec4(worldLightPosition,1)).xyz;
	cameraLightDirection = cameraLightPosition + cameraEyeDirection;
	cameraNormal = ( viewMatrix * modelMatrix * vec4(normal,0)).xyz; 
	gl_Position = clipSpacePos;

	//---spotlight---
	//hardcoded lightsource
	vec3 worldLightPositionSpot = vec3(-6.5f, 2.0f, -4.0f);
	vec3 cameraLightPositionSpot = ( viewMatrix * vec4(worldLightPositionSpot,1)).xyz;
	cameraLightDirectionSpot = cameraLightPositionSpot + cameraEyeDirection;	
	vec3 spotPos = (viewMatrix * vec4(vec3(-6.5f, 0.0f, -4.0f),1)).xyz;
	spotDir = spotPos - cameraLightPositionSpot;	

}